// ------------------------------------------------------------------------------------
// JavaORB 2.0
//
//									CosNaming Browser
//
// (c) 1998, 1999 by Jerome DANIEL
// ------------------------------------------------------------------------------------


import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * This class is a console that provides a browser for the Naming service.
 * 
 * @author	Jerome Daniel
 * @version	1.0
 */
public class Console
{
	// -----------------------------------------------------------------------
	//
	// Attributes
	//
	// -----------------------------------------------------------------------

	/**
	 * Reference to the frame
	 */
	protected JFrame frame;

	/**
	 * Reference to the tree
	 */
	protected JTree tree;

	/**
	 * Reference to the tree model
	 */
	protected DefaultTreeModel treeModel;
	
	/**
	 * Reference to the ORB
	 */
	private org.omg.CORBA.ORB orb;
	
	/**
	 * Reference to the command line arguments
	 */
	private static String [] _args;

	// -----------------------------------------------------------------------
	//
	// Constructor
	//
	// -----------------------------------------------------------------------

	/**
	 * Constructor
	 */
	public Console()
	{
		// ===============================
		//        Console creation
		// ===============================

		JPanel panel = new JPanel(true);
		frame = new JFrame("CosNaming Console");
		frame.getContentPane().add("Center", panel);
		frame.setJMenuBar( ConsoleMenu() );
		frame.setSize( 400, 400 );

		// Get naming graph from CosNaming
		ConsoleNamingContext root = loadNamingGraph();

		// Build the tree
		tree = new JTree( root );
		treeModel = ( DefaultTreeModel )tree.getModel();

		// Add the tree to the scroll pane
		JScrollPane sp = new JScrollPane();
		sp.setPreferredSize( new java.awt.Dimension(300,300) );
		sp.getViewport().add(tree);

		panel.setLayout( new java.awt.BorderLayout() );
		panel.add("Center",sp);

		// Add an event listener for this window
		frame.addWindowListener( new ConsoleAction() );

		// And show the console
		java.awt.Dimension screenSize = frame.getToolkit().getScreenSize();
        frame.setLocation((screenSize.width - frame.getSize().width) / 2 , (screenSize.height - frame.getSize().height) / 2);
        
	    frame.show();	
	}

	// -----------------------------------------------------------------------
	//
	// Console managment
	//
	// -----------------------------------------------------------------------

	/**
	 * Return the selected node	 
	 */
	private TreeNode getSelectedNode()
	{
		TreePath path = tree.getSelectionPath();

		if ( path != null )
			return ( TreeNode ) path.getLastPathComponent();

		return null;
	}

	/**
	 * Build the console menu	
	 */
	private JMenuBar ConsoleMenu()
	{
		JMenu menu;
		JMenuBar menuBar = new JMenuBar();
		JMenuItem menuItem;

		
		// Add Console entry		
		menu = new JMenu("Console");
		menuBar.add(menu);

		menuItem = menu.add( new JMenuItem("Quit") );
		menuItem.addActionListener( new EndAction() );
		
		// Add Naming Context entry
		menu = new JMenu("Naming Context");
		menuBar.add(menu);

		menuItem = menu.add( new JMenuItem("Add") );
		menuItem.addActionListener( new NCAddAction() );
		menuItem = menu.add( new JMenuItem("Remove") );
		menuItem.addActionListener( new NCRemoveAction() );
		menuItem = menu.add( new JMenuItem("Edit") );
		menuItem.addActionListener( new NCEditAction() );
		
		// Add Object entry
		menu = new JMenu("Object");
		menuBar.add(menu);

		menuItem = menu.add( new JMenuItem("Add") );
		menuItem.addActionListener( new ObjectAddAction() );
		menuItem = menu.add( new JMenuItem("Remove") );
		menuItem.addActionListener( new ObjectRemoveAction() );
		menuItem = menu.add( new JMenuItem("Edit") );
		menuItem.addActionListener( new ObjectEditAction() );
		menuItem = menu.add( new JMenuItem("Insert from a file") );
		menuItem.addActionListener( new ObjectInsertAction() );
	
		return menuBar;
	}

	/**
	 * Event managment for NamingContext/Add	 
	 */
	class NCAddAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleNamingContext c = null;

			// Get all information for a naming context
			NamingContextDialog ncd = new NamingContextDialog(frame,"NamingContext creation");
			ncd.show();

			if ( ncd.cancelButton() == true )
				return;

			// Get the selected node to add the new naming context
			try
			{
				c = ( ConsoleNamingContext ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{
				// Bad node type
				JOptionPane.showMessageDialog(null,"You must select a NamingContext","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a node ?
			if ( c != null )
			{
				// Create the new naming context
				ConsoleNamingContext nc = new ConsoleNamingContext(ncd.getName(),c);
				treeModel.insertNodeInto(nc,c,treeModel.getChildCount(c));
				
				// Add a new element to the tree										
				org.omg.CosNaming.NameComponent newNC = new org.omg.CosNaming.NameComponent();
				newNC.id = ncd.getName();
				newNC.kind = ncd.getType();				

				// Update the CosNaming for new naming context
				org.omg.CosNaming.NamingContext nctx = c.getNamingContext();
				org.omg.CosNaming.NamingContext newctx = null;
				try
				{
					org.omg.CosNaming.NameComponent [] name = new org.omg.CosNaming.NameComponent[1];
					name[0] = newNC;

					newctx = nctx.bind_new_context( name );

					nc.setNamingContext( newctx, newNC );
				}
				catch ( java.lang.Exception ex )
				{
					JOptionPane.showMessageDialog(null,"Unable to add the new NamingContext to CosNaming","Error",JOptionPane.ERROR_MESSAGE);
				}
				
			}			
		}
	}

	/**
	 * Event managment for NamingContext/Remove
	 */
	class NCRemoveAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleNamingContext c = null;
			
			// Get the selected node
			try
			{
				c = ( ConsoleNamingContext ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{				
				// Bad node type 
				JOptionPane.showMessageDialog(null,"You must select a NamingContext","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a node ?
			if ( c != null )
			{
				if ( c == ( ConsoleNamingContext ) treeModel.getRoot() )
					JOptionPane.showMessageDialog(null,"You cannot remove the CosNaming Root","Error",JOptionPane.ERROR_MESSAGE);
				else
				{
					if ( JOptionPane.showConfirmDialog(null, "Are you sure to remove "+c.getName().id, "Remove a NamingContext", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION )
					{					
						try
						{
							c.getNamingContext().destroy();
						}
						catch ( org.omg.CosNaming.NamingContextPackage.NotEmpty not_empty )
						{
							JOptionPane.showMessageDialog(null,"A NamingContext must be empty to be removed","Error",JOptionPane.ERROR_MESSAGE);
						}
						catch ( java.lang.Exception ex )
						{
							JOptionPane.showMessageDialog(null,"Unable to remove the NamingContext from CosNaming","Error",JOptionPane.ERROR_MESSAGE);
						}
						finally
						{
							treeModel.removeNodeFromParent( c );
						}
					}
				}
			}			
		}
	}

	/**
	 * Event managment for NamingContext/Edit
	 */
	class NCEditAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleNamingContext c = null;
			
			// Get the selected node
			try
			{
				c = ( ConsoleNamingContext ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{
				// Bad node kind
				JOptionPane.showMessageDialog(null,"You must select a NamingContext","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a node ?
			if ( c != null )
			{
				// Get the new Naming context information
				NamingContextDialog ncd = new NamingContextDialog(frame,"Edit a NamingContext", c.getName().id, c.getName().kind );
				ncd.show();					
			}			
		}	
	}

	/**	 
	 * Event managment for Object/Add
	 */
	class ObjectAddAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleNamingContext c = null;

			// Get the new Object information
			ObjectDialog ncd = new ObjectDialog(frame,"Object creation");
			ncd.show();

			if ( ncd.cancelButton() == true )
				return;

			// Get the selected node
			try
			{
				c = ( ConsoleNamingContext ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{
				// Bad node kind
				JOptionPane.showMessageDialog(null,"You must select a NamingContext","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a selected node ?
			if ( c != null )
			{
				// Create the new node
				ConsoleObject no = new ConsoleObject(ncd.getName(),c);
				treeModel.insertNodeInto(no,c,treeModel.getChildCount(c));
				
				// Add the new element to the CosNaming
				org.omg.CosNaming.NameComponent newNC = new org.omg.CosNaming.NameComponent();
				newNC.id = ncd.getName();
				newNC.kind = ncd.getType();

				JavaORB.util.IORBag bag = new JavaORB.util.IORBag();

				bag.version = new org.omg.GIOP.Version();

				int idx = ncd.getVersion().indexOf('.');
				if ( idx == -1 )
				{						
					bag.version.major = ( byte ) ( new Integer(ncd.getVersion()) ).intValue();
					bag.version.minor = 0;
				}
				else
				{
					String major = ncd.getVersion().substring(0,idx);
					String minor = ncd.getVersion().substring(idx+1, ncd.getVersion().length() );
					bag.version.major = ( byte ) ( new Integer(major) ).intValue();
					bag.version.minor = ( byte ) ( new Integer(minor) ).intValue();
				}

				bag.host = ncd.getHost();
				bag.port = ( short ) ( new Integer(ncd.getPort()) ).intValue();
				bag.id = ncd.getId();					
				bag.key = new byte[ ncd.getKey().length() ];
				ncd.getKey().getBytes(0,ncd.getKey().length(), bag.key ,0);		

				JavaORB.util.IORManager manager = new JavaORB.util.IORManager();

				org.omg.CORBA.Object obj = manager.bag_to_ior( bag );

				no.setObject( obj, newNC );

				// Update the CosNaming
				org.omg.CosNaming.NamingContext nctx = c.getNamingContext();					
				try
				{
					org.omg.CosNaming.NameComponent [] name = new org.omg.CosNaming.NameComponent[1];
					name[0] = newNC;

					nctx.bind( name, obj );						
				}
				catch ( java.lang.Exception ex )
				{
					JOptionPane.showMessageDialog(null,"Unable to add the new object to the CosNaming","Error",JOptionPane.ERROR_MESSAGE);
				}
			}			
		}	
	}

	/**
	 * Event managment for Object/Remove
	 */
	class ObjectRemoveAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleObject c = null;
			
			// Get the selected node
			try
			{
				c = ( ConsoleObject ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{				
				// Bad node kind
				JOptionPane.showMessageDialog(null,"You must select an Object","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a selected node ?
			if ( c != null )
			{
				if ( JOptionPane.showConfirmDialog(null, "Are you sure to remove "+c.toString(), "Remove an Object", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION )
				{					
					try
					{
						ConsoleNamingContext nctx = ( ConsoleNamingContext ) c.getParent();

						org.omg.CosNaming.NameComponent [] name = new org.omg.CosNaming.NameComponent[1];
						name[0] = c.getName();

						nctx.getNamingContext().unbind( name );
					}
					catch ( java.lang.Exception ex )
					{
						JOptionPane.showMessageDialog(null,"Unable to remove the Object from the CosNaming","Error",JOptionPane.ERROR_MESSAGE);
					}
					finally
					{
						treeModel.removeNodeFromParent( c );					
					}
				}
			}			
		}
		
	}

	/**
	 * Event managment for Object/Edit
	 */
	class ObjectEditAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleObject c = null;
			
			// Get the selected node
			try
			{
				c = ( ConsoleObject ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{
				// Bad node kind
				JOptionPane.showMessageDialog(null,"You must select an Object","Error",JOptionPane.ERROR_MESSAGE);
			}

			// Check if a node is selected
			if ( c != null )
			{

				JavaORB.util.IORManager manager = new JavaORB.util.IORManager();

				JavaORB.util.IORBag bag = manager.extract( c.getObject() );

				// Get the new object information
				ObjectDialog ncd = new ObjectDialog(frame,
													"Edit an Object", 
													c.getName().id,
													c.getName().kind,
													bag.version.major+"."+bag.version.minor,
													bag.host,
													(new Integer(bag.port)).toString(),
													bag.id,
													new String(bag.key) );
				
				ncd.show();
				
				if ( ncd.cancelButton() == true )
					return;

				if ( ! ( ( ncd.getVersion().equals( bag.version.major+"."+bag.version.minor ) ) &&
						 ( ncd.getHost().equals( bag.host ) ) &&
						 ( ncd.getPort().equals( ( new Integer(bag.port) ).toString() ) ) &&
						 ( ncd.getId().equals( bag.id ) ) &&
						 ( ncd.getKey().equals( new String(bag.key) ) ) ) )
				{

					bag.version = new org.omg.GIOP.Version();

					int idx = ncd.getVersion().indexOf('.');
					if ( idx == -1 )
					{						
						bag.version.major = ( byte ) ( new Integer(ncd.getVersion()) ).intValue();
						bag.version.minor = 0;
					}
					else
					{
						String major = ncd.getVersion().substring(0,idx);
						String minor = ncd.getVersion().substring(idx+1, ncd.getVersion().length() );
						bag.version.major = ( byte ) ( new Integer(major) ).intValue();
						bag.version.minor = ( byte ) ( new Integer(minor) ).intValue();
					}

					bag.host = ncd.getHost();
					bag.port = ( short ) ( new Integer(ncd.getPort()) ).intValue();
					bag.id = ncd.getId();					
					bag.key = new byte[ ncd.getKey().length() ];
					ncd.getKey().getBytes(0,ncd.getKey().length(), bag.key ,0);		

					org.omg.CORBA.Object obj = manager.bag_to_ior( bag );
					
					c.setObject( obj, c.getName() );

					try
					{
						ConsoleNamingContext nctx = ( ConsoleNamingContext ) c.getParent();

						org.omg.CosNaming.NameComponent [] name = new org.omg.CosNaming.NameComponent[1];
						name[0] = c.getName();

						nctx.getNamingContext().rebind( name, obj );
					}
					catch ( java.lang.Exception ex )
					{
						JOptionPane.showMessageDialog(null,"Unable to modify the Object into CosNaming","Error",JOptionPane.ERROR_MESSAGE);		
					}
				}

			}			
		}	
	}
	
	/**	 
	 * Event managment for Object/Insert
	 */
	class ObjectInsertAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			ConsoleNamingContext c = null;

			// Get the new Object information
			ObjectInsertDialog ncd = new ObjectInsertDialog(frame,"Object insertion");
			ncd.show();

			if ( ncd.cancelButton() == true )
				return;

			// Get the selected node
			try
			{
				c = ( ConsoleNamingContext ) getSelectedNode();
			}
			catch ( ClassCastException ex )
			{
				// Bad node kind
				JOptionPane.showMessageDialog(null,"You must select a NamingContext","Error",JOptionPane.ERROR_MESSAGE);
			}
			
			// Is a selected node ?
			if ( c != null )
			{
				// Create the new node
				ConsoleObject no = new ConsoleObject(ncd.getName(),c);
				treeModel.insertNodeInto(no,c,treeModel.getChildCount(c));
				
				// Add the new element to the CosNaming
				org.omg.CosNaming.NameComponent newNC = new org.omg.CosNaming.NameComponent();
				newNC.id = ncd.getName();
				newNC.kind = ncd.getType();

				org.omg.CORBA.Object obj = null;
				try
				{			
					java.io.FileInputStream file = new java.io.FileInputStream( ncd.getPath() );
					java.io.DataInputStream myInput = new java.io.DataInputStream(file);
					String stringTarget = myInput.readLine();
					
					obj = orb.string_to_object(stringTarget);
				}
				catch ( java.io.IOException ex )
				{
					JOptionPane.showMessageDialog(null,"Unable to open the IOR File","Error",JOptionPane.ERROR_MESSAGE);		
					return;
				}

				no.setObject( obj, newNC );

				// Update the CosNaming
				org.omg.CosNaming.NamingContext nctx = c.getNamingContext();					
				try
				{
					org.omg.CosNaming.NameComponent [] name = new org.omg.CosNaming.NameComponent[1];
					name[0] = newNC;

					nctx.bind( name, obj );						
				}
				catch ( java.lang.Exception ex )
				{
					JOptionPane.showMessageDialog(null,"Unable to add the new object to the CosNaming","Error",JOptionPane.ERROR_MESSAGE);
				}
			}			
		}	
	}

	/**
	 * Event managment for End
	 */
	class EndAction extends Object implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			System.exit(0);
		}
	}

	/**
	 * Event managment for the console
	 */
	class ConsoleAction extends WindowAdapter
	{
		public void windowClosing( WindowEvent e )
		{
			System.exit(0);
		}
	}

	// -----------------------------------------------------------------------
	//
	// Method for CosNaming invocations
	//
	// -----------------------------------------------------------------------

	/**
	 * Get a naming graph branch
	 */
	private void getGraphBranch( org.omg.CosNaming.NamingContext branch, ConsoleNamingContext NCnode )
	{
		org.omg.CosNaming.BindingListHolder bl = new org.omg.CosNaming.BindingListHolder();
		org.omg.CosNaming.BindingIteratorHolder bi = new org.omg.CosNaming.BindingIteratorHolder();

		org.omg.CosNaming.BindingHolder b = new org.omg.CosNaming.BindingHolder();

		try
		{
			branch.list(0,bl,bi);
			
			if ( bi.value != null )
			while ( bi.value.next_one(b) )
			{				
				System.out.print(".");

				if ( b.value.binding_type == org.omg.CosNaming.BindingType.ncontext )
				{					
					ConsoleNamingContext node = NCnode.addNamingContext( b.value.binding_name[0].id );
	
					org.omg.CORBA.Object obj = branch.resolve ( b.value.binding_name );

					org.omg.CosNaming.NamingContext newBranch = org.omg.CosNaming.NamingContextHelper.narrow(obj);

					node.setNamingContext( newBranch, b.value.binding_name[0] );

					getGraphBranch( newBranch, node );
				}
				else
				{
					ConsoleObject node = NCnode.addObject( b.value.binding_name[0].id );

					node.setObject( branch.resolve( b.value.binding_name ), b.value.binding_name[0] );
				}					
			}			
		}
		catch ( java.lang.Exception ex )
		{ 
			ex.printStackTrace();
		}
	}

	/**
	 * Get the CosNaming graph
	 */
	private ConsoleNamingContext loadNamingGraph()
	{
		
		org.omg.CORBA.Object obj = null;

		orb = org.omg.CORBA.ORB.init(_args, null);				

		ConsoleNamingContext rootNode = new ConsoleNamingContext("CosNaming Root");		 
		org.omg.CosNaming.NameComponent newNC = new org.omg.CosNaming.NameComponent();
		newNC.id = "CosNaming Root";
		newNC.kind = "JavaORB version 2.0";				
				
		try
		{
			obj = orb.resolve_initial_references("NameService");
		}
		catch ( org.omg.CORBA.ORBPackage.InvalidName ex )
		{
			System.out.println("Java ORB init reference file missing or data incorrect");
			System.exit(0);
		}
		
		
		org.omg.CosNaming.NamingContext naming = org.omg.CosNaming.NamingContextHelper.narrow(obj);

		rootNode.setNamingContext( naming, newNC );

		System.out.println("Please wait, i load the CosNaming graph");

		getGraphBranch(naming,rootNode);		

		System.out.println("");
		System.out.println("Naming graph loaded.");

		return rootNode;
	}

	// -----------------------------------------------------------------------
	//
	// Entry point for the Console Application
	//
	// -----------------------------------------------------------------------

	/**
	 * Entry point
	 */
	static public void main( String [] args )
	{
		System.out.println("Java ORB CosNaming Console");
		System.out.println("version 2.0");
		System.out.println("(c) 1998, 1999 by Jerome Daniel");
		System.out.println("");

		_args = args;
		
		new Console();
		
		System.out.println("");		
	}


}